
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using VidGrab;

namespace PlaylistWithoutGapsFromListbox
{
    public partial class Playlist_without_gaps_from_Listbox : Form
    {

        public const string AUDIO_CLIPS = "*.mp3;*.wav;*.wma;*.mid";
        public const string VIDEO_CLIPS = "*.mp*;*.vro;*.avi;*.asf;*.wmv;*.vob;*.avs;*.mov;*.divx;*.mp4;*.mts;*.3gp;*.swf;*.m2v;*.mkv;*.flv;*.wemb;*.ts;*.m4v;*.mp4v";
        public const string IMAGE_FILES = "*.jpg;*.jpeg;*.jpe;*.bmp;*.gif;*.png";
        public const string OPEN_MEDIA_FILES = "All media files|" + VIDEO_CLIPS + AUDIO_CLIPS + IMAGE_FILES
                        + "|Video clips|" + VIDEO_CLIPS
                        + "|Audio clips|" + AUDIO_CLIPS;
        public const string OPEN_MEDIA_FILES_EXTENDED = OPEN_MEDIA_FILES
                        + "|Image files|" + IMAGE_FILES;
        public const string OPEN_PICTURE_FILES = "Image files|*.bmp;*.jpg;*.gif;*.png;*.tif;*.tiff;*.wmf;*.emf;*.exf;*.jpe;*.jpeg";
        public const string OPEN_PROFILES = "WMV9 profiles (*.prx)|*.prx";
        public const string OPEN_COMPRESSOR_SETTINGS = "Compressor saved settings (*.txt)|*.txt";

        public Playlist_without_gaps_from_Listbox()
        {
            InitializeComponent();
        }

        //__________________________________________________________________________________________
        private void btnOpenPlayer_Click(object sender, EventArgs e)
        {
            lstPlayList.SelectedIndex = -1;
            playListControl1.Play(lstPlayList.Items, chkLoop.Checked);
        }

        //__________________________________________________________________________________________
        private void btnClosePlaylist_Click(object sender, EventArgs e)
        {
            playListControl1.Close();
        }

        //__________________________________________________________________________________________
        private void chkFullScreen_CheckedChanged(object sender, EventArgs e)
        {
            playListControl1.FullScreen(chkFullScreen.Checked);
        }

        //__________________________________________________________________________________________
        private void playListControl1_OnConfigureVideoGrabber(object sender, PlayListControl.TOnConfigureVideoGrabberEventArgs e)
        {
            // this event is called for each next clip loaded for playback

            // THE TVIDEOGRABBER INITIALIZATION STATEMENTS CAN BE PLACED HERE (OVERLAYS, ETC...)
            // UNCOMMENT THE LINES BELOW FOR TESTING

            e.VideoGrabber.TextOverlay_Enabled = true; // overlays the clip file name
            e.VideoGrabber.TextOverlay_String = e.NextClipFileName;

            e.VideoGrabber.Display_AspectRatio = TAspectRatio.ar_Box;

            //e.VideoGrabber.FrameGrabber = TFrameGrabber.fg_Disabled; // if no overlay or capture is needed the frame grabber can be disabled

            //e.VideoGrabber.OpenPlayerAtTimePositions(30000000, -1, true, true);  // to start playing at 3 seconds
        }

        //__________________________________________________________________________________________
        private void playListControl1_OnNextClipPlaying(object sender, PlayListControl.TOnNextClipEventArgs e)
        {
            if ((e.NextClipIndex >= 0) && (e.NextClipIndex < lstPlayList.Items.Count))
            {
                lstPlayList.SelectedIndex = e.NextClipIndex;
            }
        }

        //__________________________________________________________________________________________
        private void btnPlaylistAdd_Click(object sender, EventArgs e)
        {
            OpenFileDialog fd = new OpenFileDialog();

            fd.Filter = OPEN_MEDIA_FILES_EXTENDED;
            fd.Multiselect = true;
            if (fd.ShowDialog() == DialogResult.OK)
            {
                for (int i = 0; i < fd.FileNames.Length; i++)
                {
                    lstPlayList.Items.Add(fd.FileNames.GetValue(i));
                    lstPlayList.SelectedIndex = 0;
                }
            }
        }

        //__________________________________________________________________________________________
        private void btnCaptureToControl_Click(object sender, EventArgs e)
        {
            VideoGrabber CurrentVideoGrabberPlaying = playListControl1.CurrentVideoGrabberPlaying();
            if (CurrentVideoGrabberPlaying != null)
            {
                System.IntPtr FrameBitmap = CurrentVideoGrabberPlaying.GetLastFrameAsHBITMAP(0, false, 0, 0, 0, 0, imgCapturedFrame.Width, imgCapturedFrame.Height, 32);
                if (!FrameBitmap.Equals(System.IntPtr.Zero))
                {
                    imgCapturedFrame.Image = Image.FromHbitmap(FrameBitmap);
                    WinUtil.DeleteObject(FrameBitmap);
                }
            }
        }

        //__________________________________________________________________________________________
        private void btnCaptureToPng_Click(object sender, EventArgs e)
        {
            VideoGrabber CurrentVideoGrabberPlaying = playListControl1.CurrentVideoGrabberPlaying();
            if (CurrentVideoGrabberPlaying != null)
            {
                System.IntPtr FrameBitmap = CurrentVideoGrabberPlaying.GetLastFrameAsHBITMAP(0, false, 0, 0, 0, 0, 0, 0, 32);
                if (!FrameBitmap.Equals(System.IntPtr.Zero))
                {
                    Image img = Image.FromHbitmap(FrameBitmap);
                    img.Save("CapturedFrame.png", System.Drawing.Imaging.ImageFormat.Png);
                    WinUtil.DeleteObject(FrameBitmap);
                }
            }
        }
    }
}

